@@ -2,7 +2,7 @@ package Tibco::Rv::Msg;
 
 
 use vars qw/ $VERSION /;
-$VERSION = '0.99';
+$VERSION = '1.00';
 
 
 use constant FIELDNAME_MAX => 127;
@@ -39,11 +39,6 @@ use constant F64ARRAY => 45;
 
 use constant XML => 47;
 
-use constant USER_FIRST => 128;
-use constant USER_LAST => 255;
-
-use constant NO_TAG => 0;
-
 
 use Tibco::Rv::Msg::Field;
 
@@ -51,30 +46,34 @@ use Tibco::Rv::Msg::Field;
 use overload '""' => 'toString';
 
 
-my ( %defaults );
-BEGIN { %defaults = ( sendSubject => undef, replySubject => undef ) }
-
-
 sub new
 {
-   my ( $proto ) = @_;
+   my ( $proto ) = shift;
+   my ( %params ) = ( sendSubject => undef, replySubject => undef );
+   my ( %fields ) = ( );
+   my ( %args ) = @_;
+   foreach my $field ( keys %args )
+   {
+      next if ( exists $params{$field} );
+      $fields{$field} = $args{$field};
+      delete $args{$field};
+   }
+   %params = ( %params, %args );
    my ( $class ) = ref( $proto ) || $proto;
-   my ( $self ) = $class->_new;
+   my ( $self ) = bless { id => undef, %params };
 
    my ( $status ) = Tibco::Rv::Msg_Create( $self->{id} );
    Tibco::Rv::die( $status ) unless ( $status == Tibco::Rv::OK );
+   $self->sendSubject( $params{sendSubject} )
+      if ( defined $params{sendSubject} );
+   $self->replySubject( $params{replySubject} )
+      if ( defined $params{replySubject} );
+   map { $self->addString( $_ => $fields{$_} ) } sort keys %fields;
 
    return $self;
 }
 
 
-sub _new
-{
-   my ( $class, $id ) = @_;
-   return bless { id => $id, %defaults }, $class;
-}
-
-
 sub _adopt
 {
    my ( $proto, $id ) = @_;
@@ -82,10 +81,11 @@ sub _adopt
    my ( $class ) = ref( $proto );
    if ( $class )
    {
-      $self->DESDTROY;
-      @$self{ 'id', keys %defaults } = ( $id, values %defaults );
+      $self->DESTROY;
+      @$self{ qw/ id sendSubject replySubject / } = ( $id, undef, undef );
    } else {
-      $self = bless $proto->_new( $id ), $proto;
+      $self = bless { id => $id,
+         sendSubject => undef, replySubject => undef }, $proto;
    }
    $self->_getValues;
    return $self;
@@ -117,7 +117,8 @@ sub createFromBytes
 {
    my ( $proto, $bytes ) = @_;
    my ( $class ) = ref( $proto ) || $proto;
-   my ( $self ) = $class->_new;
+   my ( $self ) = bless { id => undef,
+      sendSubject => undef, replySubject => undef }, $class;
 
    my ( $status ) = Tibco::Rv::Msg_CreateFromBytes( $self->{id}, $bytes );
    Tibco::Rv::die( $status ) unless ( $status == Tibco::Rv::OK );
@@ -159,7 +160,7 @@ sub reset
 {
    my ( $self ) = @_;
    my ( $status ) = Tibco::Rv::tibrvMsg_Reset( $self->{id} );
-   @$self{ keys %defaults } = values %defaults;
+   @$self{ qw/ sendSubject replySubject / } = ( undef, undef );
    Tibco::Rv::die( $status ) unless ( $status == Tibco::Rv::OK );
 }
 
@@ -406,7 +407,7 @@ sub removeField
       Tibco::Rv::tibrvMsg_RemoveFieldEx( $self->{id}, $fieldName, $fieldId );
    Tibco::Rv::die( $status )
       unless ( $status == Tibco::Rv::OK or $status == Tibco::Rv::NOT_FOUND );
-   return new Tibco::Rv::Status( $status );
+   return new Tibco::Rv::Status( status => $status );
 }
 
 
@@ -417,7 +418,7 @@ sub removeFieldInstance
       $fieldName, $instance );
    Tibco::Rv::die( $status )
       unless ( $status == Tibco::Rv::OK or $status == Tibco::Rv::NOT_FOUND );
-   return new Tibco::Rv::Status( $status );
+   return new Tibco::Rv::Status( status => $status );
 }
 
 
@@ -572,3 +573,342 @@ sub DESTROY
 
 
 1;
+
+
+=pod
+
+=head1 NAME
+
+Tibco::Rv::Msg - Tibco message object
+
+=head1 SYNOPSIS
+
+   my ( $rv ) = new Tibco::Rv;
+   my ( $msg ) = $rv->createMsg;
+
+   $msg->addString( myField => 'a string' );
+   $msg->addBool( myField2 => Tibco::Rv::TRUE );
+   $msg->addI8Array( myNums => [ 1, 2, 3 ] );
+
+   $msg->sendSubject( 'MY.SEND.SUBJECT' );
+   $rv->send( $msg );
+
+=head1 DESCRIPTION
+
+Tibco Message-manipulating class.  Add/update/delete data fields, set
+subject addressing information, and get the on-the-wire byte representation.
+
+All methods die with a L<Tibco::Rv::Status|Tibco::Rv::Status> message if
+there are any TIB/Rendezvous errors.
+
+=head1 CONSTRUCTOR
+
+=over 4
+
+=item $msg = new Tibco::Rv::Msg( %args )
+
+   %args:
+      sendSubject => $sendSubject,
+      replySubject => $replySubject,
+      $fieldName1 => $stringValue1,
+      $fieldName2 => $stringValue2, ...
+
+Creates a C<Tibco::Rv::Msg>, with sendSubject and replySubject as given
+in %args (sendSubject and replySubject default to C<undef> if not specified).
+Any other name => value pairs are added as string fields.
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item $field = $msg->createField
+
+Returns a new L<Tibco::Rv::Msg::Field|Tibco::Rv::Msg::Field> object.
+
+=item $date = $msg->createDateTime
+
+Returns a new L<Tibco::Rv::Msg::DateTime|Tibco::Rv::Msg::DateTime> object.
+
+=item $msgCopy = $msg->copy
+
+Returns a newly created, independent copy of C<$msg>.  C<$msgCopy> has all
+the same field data as C<$msg>, but none of the subject addressing information.
+
+=item $bytes = $msg->bytes
+
+Returns the on-the-wire byte representation of C<$msg> as a scalar value.
+
+=item $bytes = $msg->bytesCopy
+
+Same as C<bytes>, but with an extraneous memory allocation.  You probably just
+want to use C<bytes>.
+
+=item $msg = Tibco::Rv::Msg->createFromBytes( $bytes )
+
+Returns a newly created C<Tibco::Rv::Msg> from the on-the-wire byte
+representation C<$bytes>.
+
+=item $msg->expand( $additionalStorage )
+
+Increase memory allocated for this message by C<$addtionalStorage> bytes.
+You might want to do this before adding a lot of data to a message.
+
+=item $msg->reset
+
+Removes all fields and subject addressing information.
+
+=item $numFields = $msg->numFields
+
+Returns the number of fields in C<$msg> (not including fields in sub-messages).
+
+=item $byteSize = $msg->byteSize
+
+Returns the number of bytes taken up by the on-the-wire byte representation.
+
+=item $string = $msg->toString (or "$msg")
+
+Returns a string representation of C<$msg> for printing.
+
+=item $subject = $msg->sendSubject
+
+Returns the subject on which C<$msg> will be published when sent via a
+Transport object.
+
+=item $msg->sendSubject( $subject )
+
+Sets the subject on which C<$msg> will be published went sent via a
+Transport object.  Returns the new subject.
+
+=item $subject = $msg->replySubject
+
+Returns the subject on which replies will be received when C<$msg> is sent
+as a request/reply message via a Transport's sendRequest method.
+
+=item $msg->replySubject( $subject )
+
+Sets the subject on which replies will be recieved when C<$msg> is sent
+as a request/reply message via a Transport's sendRequest method.  Returns
+the new subject.
+
+=item $msg->addField( $field )
+
+Adds L<Field|Tibco::Rv::Msg::Field> C<$field> to C<$msg>.
+
+=item $msg->add<type>( $fieldName => $value, $fieldId )
+
+   <type> can be:
+      Bool, String, Opaque, Xml,
+      F32, F64, I8, I16, I32, I64, U8, U16, U32, U64,
+      IPAddr32, IPPort16, DateTime, or Msg
+
+Adds C<$value> to C<$msg> at field C<$fieldName>, as type E<lt>typeE<gt>.
+C<$fieldId> is an optional field identifier.  It must be unique with this
+message.
+
+Bool values should be Tibco::Rv::TRUE or Tibco::Rv::FALSE.
+
+Opaque values can contain embedded nulls ('\0'); String and Xml values
+cannot (and if you try, they'll be truncated to the first null).
+
+IPAddr32 values should be specified in dotted-quad notation.  For example,
+'66.35.250.150'.
+
+DateTime values must be of type
+L<Tibco::Rv::Msg::DateTime|Tibco::Rv::Msg::DateTime>.
+
+=item $msg->add<type>Array( $fieldName => [ $val1, $val2, ... ], $fieldId )
+
+   <type> can be:
+   F32, F64, I8, I16, I32, I64, U8, U16, U32, or U64
+
+Adds the given array reference of E<lt>typeE<gt> values to C<$msg> at field
+C<$fieldName>.  C<$fieldId> is an optional field identifier.  It must be
+unique within this message.
+
+=item $value = $msg->get<type>( $fieldName, $fieldId )
+
+   <type> can be:
+      Field, Bool, String, Opaque, Xml,
+      F32, F64, I8, I16, I32, I64, U8, U16, U32, U64,
+      IPAddr32, IPPort16, DateTime, or Msg
+
+Returns the value of the specified field.  If C<$fieldId> is not specified
+(or C<undef>), returns the first field found named C<$fieldName>.  If
+C<$fieldId> is specified, returns the field with the given C<$fieldId>.
+
+If the specified field is not found, returns C<undef>.
+
+If the field is found but it is of a different type, returns the value
+converted to the given E<lt>typeE<gt>.  If conversion is not possible,
+dies with a Tibco::Rv::CONVERSION_FAILED Status message.
+
+If C<$fieldId> is specified but is not found, and a field named C<$fieldName>
+is found but with a different C<$fieldId>, then this method dies with
+a Tibco::Rv::ID_CONFLICT Status message.
+
+=item $valueAryRef = $msg->get<type>Array( $fieldName, $fieldId )
+
+   <type> can be:
+      F32, F64, I8, I16, I32, I64, U8, U16, U32, U64,
+
+Behaves the same as getE<lt>typeE<gt>, except that it returns an array
+reference of values instead of a single value.
+
+=item $field = $msg->getFieldByIndex( $fieldIndex )
+
+Returns field at index C<$fieldIndex>.  Iterate over all fields in C<$msg>
+by using this method over range 0 .. numFields - 1.
+
+=item $field = $msg->getFieldInstance( $fieldName, $instance )
+
+When a message contains multiple fields with the same name, use this method
+to interate over all messages named C<$fieldName>.  Returns C<undef> if
+not found, and when C<$instance> exceeds the number of fields in this
+message named C<$fieldName>.  The first field of a given name is retrieved
+by using C<$instance = 1>.
+
+=item $status = $msg->removeField( $fieldName, $fieldId )
+
+Searches for a field using the same algorithm as getE<lt>typeE<gt>.  If
+found, removes it.  Returns Status Tibco::Rv::OK if found and deleted,
+or Status Tibco::Rv::NOT_FOUND if not found.
+
+=item $status = $msg->removeFieldInstance( $fieldName, $instance )
+
+Searches for a field using the same algorithm as getFieldInstance.  If
+found, removes it.  Returns Status Tibco::Rv::OK if found and deleted,
+or Status Tibco::Rv::NOT_FOUND if not found.
+
+=item $msg->updateField( $field )
+
+Updates the field specified by C<$field>'s name and identifier.  If the field
+is not found, then C<$field> is simply added.
+
+If the field is found in C<$msg>, and its type does not match the type of
+C<$field>, this method dies with a Tibco::Rv::INVALID_TYPE Status message.
+
+=item $msg->update<type>( $fieldName, $value, $fieldId )
+
+   <type> can be:
+      Field, Bool, String, Opaque, Xml,
+      F32, F64, I8, I16, I32, I64, U8, U16, U32, U64,
+      IPAddr32, IPPort16, DateTime, or Msg
+
+Updates the field specified by C<$fieldName> and C<$fieldId> (C<$fieldId> is
+optional).  If the field is not found, then C<$value> is simply added.
+
+If the field is found in C<$msg>, and its type does not match the type of
+C<$field>, this method dies with a Tibco::Rv::INVALID_TYPE Status message.
+
+=item $msg->update<type>Array( $fieldName, [ $val1, $val2, ... ], $fieldId )
+
+   <type> can be:
+      F32, F64, I8, I16, I32, I64, U8, U16, U32, U64,
+
+Behaves the same as update<lt>typeE<gt>, except that it takes an array
+reference of values instead of a single value.
+
+=item $msg->markReferences
+
+See TIB/Rendezvous documentation for discussion on what this method does.
+
+=item $msg->clearReferences
+
+See TIB/Rendezvous documentation for discussion on what this method does.
+
+=back
+
+=head1 MESSAGE TYPE CONSTANTS
+
+=over 4
+
+=item Tibco::Rv::Msg::MSG => 1
+
+
+=item Tibco::Rv::Msg::DATETIME => 3
+
+=item Tibco::Rv::Msg::OPAQUE => 7
+
+=item Tibco::Rv::Msg::STRING => 8
+
+=item Tibco::Rv::Msg::BOOL => 9
+
+=item Tibco::Rv::Msg::I8 => 14
+
+=item Tibco::Rv::Msg::U8 => 15
+
+=item Tibco::Rv::Msg::I16 => 16
+
+=item Tibco::Rv::Msg::U16 => 17
+
+=item Tibco::Rv::Msg::I32 => 18
+
+=item Tibco::Rv::Msg::U32 => 19
+
+=item Tibco::Rv::Msg::I64 => 20
+
+=item Tibco::Rv::Msg::U64 => 21
+
+=item Tibco::Rv::Msg::F32 => 24
+
+=item Tibco::Rv::Msg::F64 => 25
+
+=item Tibco::Rv::Msg::IPPORT16 => 26
+
+=item Tibco::Rv::Msg::IPADDR32 => 27
+
+=item Tibco::Rv::Msg::ENCRYPTED => 32
+
+=item Tibco::Rv::Msg::NONE => 22
+
+=item Tibco::Rv::Msg::I8ARRAY => 34
+
+=item Tibco::Rv::Msg::U8ARRAY => 35
+
+=item Tibco::Rv::Msg::I16ARRAY => 36
+
+=item Tibco::Rv::Msg::U16ARRAY => 37
+
+=item Tibco::Rv::Msg::I32ARRAY => 38
+
+=item Tibco::Rv::Msg::U32ARRAY => 39
+
+=item Tibco::Rv::Msg::I64ARRAY => 40
+
+=item Tibco::Rv::Msg::U64ARRAY => 41
+
+=item Tibco::Rv::Msg::F32ARRAY => 44
+
+=item Tibco::Rv::Msg::F64ARRAY => 45
+
+=item Tibco::Rv::Msg::XML => 47
+
+=back
+
+=head1 OTHER CONSTANTS
+
+=over 4
+
+=item Tibco::Rv::Msg::FIELDNAME_MAX => 127
+
+Maximum length of a field name
+
+=back
+
+=head1 SEE ALSO
+
+=over 4
+
+=item L<Tibco::Rv::Msg::Field>
+
+=item L<Tibco::Rv::Msg::DateTime>
+
+=back
+
+=head1 AUTHOR
+
+Paul Sturm E<lt>I<sturm@branewave.com>E<gt>
+
+=cut
